@@ -197,6 +197,13 @@ static void QueueMessage(InputMessageBase* to_run_msg,
     if (!to_run_msg) {
         return;
     }
+
+#if BRPC_WITH_RDMA
+    if (rdma::FLAGS_rdma_disable_bthread) {
+        ProcessInputMessage(to_run_msg);
+        return;
+    }
+#endif
     // Create bthread for last_msg. The bthread is not scheduled
     // until bthread_flush() is called (in the worse case).
                 
@@ -207,14 +214,8 @@ static void QueueMessage(InputMessageBase* to_run_msg,
                           BTHREAD_ATTR_NORMAL) | BTHREAD_NOSIGNAL;
     tmp.keytable_pool = keytable_pool;
     tmp.tag = bthread_self_tag();
-
-#if BRPC_WITH_RDMA
-    if (rdma::FLAGS_rdma_disable_bthread) {
-        ProcessInputMessage(to_run_msg);
-        return;
-    }
-#endif
-
+    bthread_attr_set_name(&tmp, "ProcessInputMessage");
+    
     if (!FLAGS_usercode_in_coroutine && bthread_start_background(
             &th, &tmp, ProcessInputMessage, to_run_msg) == 0) {
         ++*num_bthread_created;
@@ -331,12 +332,7 @@ int InputMessenger::ProcessNewMessage(
                     "destroyed when authentication failed";
             }
         }
-#if BRPC_WITH_RDMA
-        if (!m->is_read_progressive() && !rdma::FLAGS_rdma_use_polling)
-#else
-        if (!m->is_read_progressive())
-#endif
-        {
+        if (!m->is_read_progressive()) {
             // Transfer ownership to last_msg
             last_msg.reset(msg.release());
         } else {
@@ -346,6 +342,16 @@ int InputMessenger::ProcessNewMessage(
             num_bthread_created = 0;
         }
     }
+#if BRPC_WITH_RDMA
+    // In RDMA polling mode, all messages must be executed in a new bthread and
+    // not in the bthread where the polling bthread is located, because the
+    // method for processing messages may call synchronization primitives,
+    // causing the polling bthread to be scheduled out.
+    if (rdma::FLAGS_rdma_use_polling) {
+        QueueMessage(last_msg.release(), &num_bthread_created,
+                     m->_keytable_pool);
+    }
+#endif
     if (num_bthread_created) {
         bthread_flush();
     }
