@@ -35,6 +35,10 @@ namespace bthread {
 
 DEFINE_uint32(signal_trace_timeout_ms, 50, "Timeout for signal trace in ms");
 BUTIL_VALIDATE_GFLAG(signal_trace_timeout_ms, butil::PositiveInteger<uint32_t>);
+// Note that SIGURG handler may be registered by some library such as cgo
+// so let the signal number be configurable
+DEFINE_int32(signal_number_for_trace, SIGURG,
+             "signal number used for stack trace, default to SIGURG");
 
 extern BAIDU_THREAD_LOCAL TaskMeta* pthread_fake_meta;
 
@@ -119,6 +123,7 @@ void TaskTracer::Result::OutputToStream(std::ostream& os) const {
 
 bool TaskTracer::Init() {
     if (_trace_time.expose("bthread_trace_time") != 0) {
+        LOG(ERROR) << "Fail to expose bthread_trace_time";
         return false;
     }
     if (!RegisterSignalHandler()) {
@@ -136,7 +141,7 @@ void TaskTracer::set_status(TaskStatus s, TaskMeta* m) {
     CHECK_NE(TASK_STATUS_RUNNING, s) << "Use `set_running_status' instead";
     CHECK_NE(TASK_STATUS_END, s) << "Use `set_end_status_unsafe' instead";
 
-    bool tracing;
+    bool tracing = false;
     {
         BAIDU_SCOPED_LOCK(m->version_lock);
         if (TASK_STATUS_UNKNOWN == m->status && TASK_STATUS_JUMPING == s) {
@@ -146,7 +151,7 @@ void TaskTracer::set_status(TaskStatus s, TaskMeta* m) {
 
         tracing = m->traced;
         // bthread is scheduled for the first time.
-        if (TASK_STATUS_READY == s || NULL == m->stack) {
+        if (TASK_STATUS_READY == s && NULL == m->stack) {
             m->status = TASK_STATUS_FIRST_READY;
         } else {
             m->status = s;
@@ -182,31 +187,8 @@ void TaskTracer::Trace(std::ostream& os, bthread_t tid) {
 }
 
 void TaskTracer::WaitForTracing(TaskMeta* m) {
-    BAIDU_SCOPED_LOCK(_mutex);
-    while (m->traced) {
-        _cond.Wait();
-    }
-}
-
-TaskStatus TaskTracer::WaitForJumping(TaskMeta* m) {
-    // Reasons for not using locks here:
-    // 1. It is necessary to lock before jump_stack, unlock after jump_stack,
-    //    which involves two different bthread and is prone to errors.
-    // 2. jump_stack is fast.
-    int i = 0;
-    do {
-        // The bthread is jumping now, spin until it finishes.
-        if (i++ < 30) {
-            cpu_relax();
-        } else {
-            sched_yield();
-        }
-
-        BAIDU_SCOPED_LOCK(m->version_lock);
-        if (TASK_STATUS_JUMPING != m->status) {
-            return m->status;
-        }
-    } while (true);
+    BAIDU_SCOPED_LOCK(m->trace_lock);
+    // Acquiring trace_lock means tracing is done.
 }
 
 TaskTracer::Result TaskTracer::TraceImpl(bthread_t tid) {
@@ -224,25 +206,44 @@ TaskTracer::Result TaskTracer::TraceImpl(bthread_t tid) {
     // Make sure only one bthread is traced at a time.
     BAIDU_SCOPED_LOCK(_trace_request_mutex);
 
+    // The chance to remove unused SignalSyncs.
+    auto iter = std::remove_if(
+        _inuse_signal_syncs.begin(), _inuse_signal_syncs.end(),
+        [](butil::intrusive_ptr<SignalSync>& sync) {
+            return sync->ref_count() == 1;
+    });
+    _inuse_signal_syncs.erase(iter, _inuse_signal_syncs.end());
+
     TaskMeta* m = TaskGroup::address_meta(tid);
     if (NULL == m) {
         return Result::MakeErrorResult("bthread=%d never existed", tid);
     }
 
-    BAIDU_SCOPED_LOCK(_mutex);
+    BAIDU_SCOPED_LOCK(m->trace_lock);
     TaskStatus status;
     pthread_t worker_tid;
     const uint32_t given_version = get_version(tid);
     {
         BAIDU_SCOPED_LOCK(m->version_lock);
-        if (given_version == *m->version_butex) {
-            // Start tracing.
-            m->traced = true;
-            worker_tid = m->worker_tid;
-            status = m->status;
-        } else {
+        if (given_version != *m->version_butex) {
             return Result::MakeErrorResult("bthread=%d not exist now", tid);
         }
+
+        status = m->status;
+        if (TASK_STATUS_UNKNOWN == status) {
+            return Result::MakeErrorResult("bthread=%d not exist now", tid);
+        } else if (TASK_STATUS_CREATED == status) {
+            return Result::MakeErrorResult("bthread=%d has just been created", tid);
+        } else if (TASK_STATUS_FIRST_READY == status) {
+            return Result::MakeErrorResult("bthread=%d is scheduled for the first time", tid);
+        } else if (TASK_STATUS_END == status) {
+            return Result::MakeErrorResult("bthread=%d has ended", tid);
+        } else if (TASK_STATUS_JUMPING == status) {
+            return Result::MakeErrorResult("bthread=%d is jumping stack", tid);
+        }
+        // Start tracing.
+        m->traced = true;
+        worker_tid = m->worker_tid;
     }
 
     BRPC_SCOPE_EXIT {
@@ -252,31 +253,16 @@ TaskTracer::Result TaskTracer::TraceImpl(bthread_t tid) {
             // tracing completion, so given_version != *m->version_butex is OK.
             m->traced = false;
         }
-        // Wake up the waiting worker thread to jump.
-        _cond.Signal();
     };
 
-    if (TASK_STATUS_UNKNOWN == status) {
-        return Result::MakeErrorResult("bthread=%d not exist now", tid);
-    } else if (TASK_STATUS_CREATED == status) {
-        return Result::MakeErrorResult("bthread=%d has just been created", tid);
-    } else if (TASK_STATUS_FIRST_READY == status) {
-        return Result::MakeErrorResult("bthread=%d is scheduled for the first time", tid);
-    } else if (TASK_STATUS_END == status) {
-        return Result::MakeErrorResult("bthread=%d has ended", tid);
-    } else if (TASK_STATUS_JUMPING == status) {
-        // Wait for jumping completion.
-        status = WaitForJumping(m);
-    }
-
-    // After jumping, the status may be RUNNING, SUSPENDED, or READY, which is traceable.
+    // The status may be RUNNING, SUSPENDED, or READY, which is traceable.
     if (TASK_STATUS_RUNNING == status) {
         return SignalTrace(worker_tid);
     } else if (TASK_STATUS_SUSPENDED == status || TASK_STATUS_READY == status) {
         return ContextTrace(m->stack->context);
     }
 
-    return Result::MakeErrorResult("Invalid TaskStatus=%d", status);
+    return Result::MakeErrorResult("Invalid TaskStatus=%d of bthread=%d", status, tid);
 }
 
 // Instruct ASan to ignore this function.
@@ -333,17 +319,19 @@ TaskTracer::Result TaskTracer::TraceByLibunwind(unw_cursor_t& cursor) {
 
 bool TaskTracer::RegisterSignalHandler() {
     // Set up the signal handler.
+    _signal_num = FLAGS_signal_number_for_trace;
     struct sigaction old_sa{};
     struct sigaction sa{};
     sa.sa_sigaction = SignalHandler;
     sa.sa_flags = SA_SIGINFO;
     sigfillset(&sa.sa_mask);
-    if (sigaction(SIGURG, &sa, &old_sa) != 0) {
+    if (sigaction(_signal_num, &sa, &old_sa) != 0) {
         PLOG(ERROR) << "Failed to sigaction";
         return false;
     }
     if (NULL != old_sa.sa_handler || NULL != old_sa.sa_sigaction) {
-        LOG(ERROR) << "Signal handler of SIGURG is already registered";
+        LOG(ERROR) << "Signal handler of signal number "
+                   << _signal_num << " is already registered";
         return false;
     }
 
@@ -408,14 +396,6 @@ TaskTracer::Result TaskTracer::SignalTrace(pthread_t worker_tid) {
         return Result::MakeErrorResult("Forbid to trace self");
     }
 
-    // Remove unused SignalSyncs.
-    auto iter = std::remove_if(
-        _inuse_signal_syncs.begin(), _inuse_signal_syncs.end(),
-        [](butil::intrusive_ptr<SignalSync>& sync) {
-            return sync->ref_count() == 1;
-    });
-    _inuse_signal_syncs.erase(iter, _inuse_signal_syncs.end());
-
     // Each signal trace has an independent SignalSync to
     // prevent the previous SignalHandler from affecting the new SignalTrace.
     butil::intrusive_ptr<SignalSync> signal_sync(new SignalSync());
@@ -429,14 +409,13 @@ TaskTracer::Result TaskTracer::SignalTrace(pthread_t worker_tid) {
     sigval value{};
     value.sival_ptr = signal_sync.get();
     size_t sigqueue_try = 0;
-    while (pthread_sigqueue(worker_tid, SIGURG, value) != 0) {
+    while (pthread_sigqueue(worker_tid, _signal_num, value) != 0) {
         if (errno != EAGAIN || sigqueue_try++ >= 3) {
             // Remove reference for SignalHandler.
             signal_sync->RemoveRefManually();
             return Result::MakeErrorResult("Fail to pthread_sigqueue: %s", berror());
         }
     }
-    _inuse_signal_syncs.push_back(signal_sync);
 
     // Wait for the signal handler to complete.
     butil::Timer timer;
@@ -461,11 +440,14 @@ TaskTracer::Result TaskTracer::SignalTrace(pthread_t worker_tid) {
             if (EINTR == errno) {
                 continue;
             }
+            // Extend the lifetime of `signal_sync' to
+            // avoid it being destroyed in the signal handler.
+            _inuse_signal_syncs.push_back(signal_sync);
             return Result::MakeErrorResult("Fail to poll: %s", berror());
         }
         break;
     }
-    
+
     return signal_sync->result;
 }
 
