@@ -183,6 +183,20 @@ butil::static_atomic<size_t> g_nblock = BUTIL_STATIC_ATOMIC_INIT(0);
 butil::static_atomic<size_t> g_blockmem = BUTIL_STATIC_ATOMIC_INIT(0);
 butil::static_atomic<size_t> g_newbigview = BUTIL_STATIC_ATOMIC_INIT(0);
 
+void inc_g_nblock() {
+    g_nblock.fetch_add(1, butil::memory_order_relaxed);
+}
+void dec_g_nblock() {
+    g_nblock.fetch_sub(1, butil::memory_order_relaxed);
+}
+
+void inc_g_blockmem() {
+    g_blockmem.fetch_add(1, butil::memory_order_relaxed);
+}
+void dec_g_blockmem() {
+    g_blockmem.fetch_sub(1, butil::memory_order_relaxed);
+}
+
 }  // namespace iobuf
 
 size_t IOBuf::block_count() {
@@ -197,133 +211,6 @@ size_t IOBuf::new_bigview_count() {
     return iobuf::g_newbigview.load(butil::memory_order_relaxed);
 }
 
-const uint16_t IOBUF_BLOCK_FLAGS_USER_DATA = 1 << 0;
-const uint16_t IOBUF_BLOCK_FLAGS_SAMPLED = 1 << 1;
-using UserDataDeleter = std::function<void(void*)>;
-
-struct UserDataExtension {
-    UserDataDeleter deleter;
-};
-
-struct IOBuf::Block {
-    butil::atomic<int> nshared;
-    uint16_t flags;
-    uint16_t abi_check;  // original cap, never be zero.
-    uint32_t size;
-    uint32_t cap;
-    // When flag is 0, portal_next is valid.
-    // When flag & IOBUF_BLOCK_FLAGS_USER_DATA is non-0, data_meta is valid.
-    union {
-        Block* portal_next;
-        uint64_t data_meta;
-    } u;
-    // When flag is 0, data points to `size` bytes starting at `(char*)this+sizeof(Block)'
-    // When flag & IOBUF_BLOCK_FLAGS_USER_DATA is non-0, data points to the user data and
-    // the deleter is put in UserDataExtension at `(char*)this+sizeof(Block)'
-    char* data;
-        
-    Block(char* data_in, uint32_t data_size)
-        : nshared(1)
-        , flags(0)
-        , abi_check(0)
-        , size(0)
-        , cap(data_size)
-        , u({NULL})
-        , data(data_in) {
-        iobuf::g_nblock.fetch_add(1, butil::memory_order_relaxed);
-        iobuf::g_blockmem.fetch_add(data_size + sizeof(Block),
-                                    butil::memory_order_relaxed);
-        if (is_samplable()) {
-            SubmitIOBufSample(this, 1);
-        }
-    }
-
-    Block(char* data_in, uint32_t data_size, UserDataDeleter deleter)
-        : nshared(1)
-        , flags(IOBUF_BLOCK_FLAGS_USER_DATA)
-        , abi_check(0)
-        , size(data_size)
-        , cap(data_size)
-        , u({0})
-        , data(data_in) {
-        auto ext = new (get_user_data_extension()) UserDataExtension();
-        ext->deleter = std::move(deleter);
-        if (is_samplable()) {
-            SubmitIOBufSample(this, 1);
-        }
-    }
-
-    // Undefined behavior when (flags & IOBUF_BLOCK_FLAGS_USER_DATA) is 0.
-    UserDataExtension* get_user_data_extension() {
-        char* p = (char*)this;
-        return (UserDataExtension*)(p + sizeof(Block));
-    }
-
-    inline void check_abi() {
-#ifndef NDEBUG
-        if (abi_check != 0) {
-            LOG(FATAL) << "Your program seems to wrongly contain two "
-                "ABI-incompatible implementations of IOBuf";
-        }
-#endif
-    }
-
-    void inc_ref() {
-        check_abi();
-        nshared.fetch_add(1, butil::memory_order_relaxed);
-        if (sampled()) {
-            SubmitIOBufSample(this, 1);
-        }
-    }
-        
-    void dec_ref() {
-        check_abi();
-        if (sampled()) {
-            SubmitIOBufSample(this, -1);
-        }
-        if (nshared.fetch_sub(1, butil::memory_order_release) == 1) {
-            butil::atomic_thread_fence(butil::memory_order_acquire);
-            if (!is_user_data()) {
-                iobuf::g_nblock.fetch_sub(1, butil::memory_order_relaxed);
-                iobuf::g_blockmem.fetch_sub(cap + sizeof(Block),
-                                            butil::memory_order_relaxed);
-                this->~Block();
-                iobuf::blockmem_deallocate(this);
-            } else if (flags & IOBUF_BLOCK_FLAGS_USER_DATA) {
-                auto ext = get_user_data_extension();
-                ext->deleter(data);
-                ext->~UserDataExtension();
-                this->~Block();
-                free(this);
-            }
-        }
-    }
-
-    int ref_count() const {
-        return nshared.load(butil::memory_order_relaxed);
-    }
-
-    bool full() const { return size >= cap; }
-    size_t left_space() const { return cap - size; }
-
-private:
-    bool is_samplable() {
-        if (IsIOBufProfilerSamplable()) {
-            flags |= IOBUF_BLOCK_FLAGS_SAMPLED;
-            return true;
-        }
-        return false;
-    }
-
-    bool sampled() const {
-        return flags & IOBUF_BLOCK_FLAGS_SAMPLED;
-    }
-
-    bool is_user_data() const {
-        return flags & IOBUF_BLOCK_FLAGS_USER_DATA;
-    }
-};
-
 namespace iobuf {
 
 // for unit test
@@ -341,23 +228,6 @@ uint32_t block_size(IOBuf::Block const* b) {
     return b->size;
 }
 
-inline IOBuf::Block* create_block(const size_t block_size) {
-    if (block_size > 0xFFFFFFFFULL) {
-        LOG(FATAL) << "block_size=" << block_size << " is too large";
-        return NULL;
-    }
-    char* mem = (char*)iobuf::blockmem_allocate(block_size);
-    if (mem == NULL) {
-        return NULL;
-    }
-    return new (mem) IOBuf::Block(mem + sizeof(IOBuf::Block),
-                                  block_size - sizeof(IOBuf::Block));
-}
-
-inline IOBuf::Block* create_block() {
-    return create_block(IOBuf::DEFAULT_BLOCK_SIZE);
-}
-
 inline IOBuf::Block* create_block_aligned(size_t block_size, size_t alignment) {
     if (block_size > 0xFFFFFFFFULL) {
         LOG(FATAL) << "block_size=" << block_size << " is too large";
@@ -376,28 +246,11 @@ inline IOBuf::Block* create_block_aligned(size_t block_size, size_t alignment) {
 }
 
 // === Share TLS blocks between appending operations ===
-// Max number of blocks in each TLS. This is a soft limit namely
-// release_tls_block_chain() may exceed this limit sometimes.
-const int MAX_BLOCKS_PER_THREAD = 8;
-
-inline int max_blocks_per_thread() {
-    // If IOBufProfiler is enabled, do not cache blocks in TLS.
-    return IsIOBufProfilerEnabled() ? 0 : MAX_BLOCKS_PER_THREAD;
-}
-
-struct TLSData {
-    // Head of the TLS block chain.
-    IOBuf::Block* block_head;
-    
-    // Number of TLS blocks
-    int num_blocks;
-    
-    // True if the remote_tls_block_chain is registered to the thread.
-    bool registered;
-};
 
 static __thread TLSData g_tls_data = { NULL, 0, false };
 
+// Used in release_tls_block()
+TLSData* get_g_tls_data() { return &g_tls_data; }
 // Used in UT
 IOBuf::Block* get_tls_block_head() { return g_tls_data.block_head; }
 int get_tls_block_count() { return g_tls_data.num_blocks; }
@@ -407,6 +260,14 @@ int get_tls_block_count() { return g_tls_data.num_blocks; }
 // of appending functions in IOPortal may be lowered.
 static butil::static_atomic<size_t> g_num_hit_tls_threshold = BUTIL_STATIC_ATOMIC_INIT(0);
 
+void inc_g_num_hit_tls_threshold() {
+    g_num_hit_tls_threshold.fetch_add(1, butil::memory_order_relaxed);
+}
+
+void dec_g_num_hit_tls_threshold() {
+    g_num_hit_tls_threshold.fetch_sub(1, butil::memory_order_relaxed);
+}
+
 // Called in UT.
 void remove_tls_block_chain() {
     TLSData& tls_data = g_tls_data;
@@ -458,28 +319,6 @@ IOBuf::Block* share_tls_block() {
     return new_block;
 }
 
-// Return one block to TLS.
-inline void release_tls_block(IOBuf::Block* b) {
-    if (!b) {
-        return;
-    }
-    TLSData& tls_data = g_tls_data;
-    if (b->full()) {
-        b->dec_ref();
-    } else if (tls_data.num_blocks >= max_blocks_per_thread()) {
-        b->dec_ref();
-        g_num_hit_tls_threshold.fetch_add(1, butil::memory_order_relaxed);
-    } else {
-        b->u.portal_next = tls_data.block_head;
-        tls_data.block_head = b;
-        ++tls_data.num_blocks;
-        if (!tls_data.registered) {
-            tls_data.registered = true;
-            butil::thread_atexit(remove_tls_block_chain);
-        }
-    }
-}
-
 // Return chained blocks to TLS.
 // NOTE: b MUST be non-NULL and all blocks linked SHOULD not be full.
 void release_tls_block_chain(IOBuf::Block* b) {
@@ -1090,16 +929,17 @@ ssize_t IOBuf::cut_multiple_into_SSL_channel(SSL* ssl, IOBuf* const* pieces,
     }
 
 #ifndef USE_MESALINK
-    // Flush remaining data inside the BIO buffer layer
-    BIO* wbio = SSL_get_wbio(ssl);
-    if (BIO_wpending(wbio) > 0) {
-        int rc = BIO_flush(wbio);
-        if (rc <= 0 && BIO_fd_non_fatal_error(errno) == 0) {
-            // Fatal error during BIO_flush
-            *ssl_error = SSL_ERROR_SYSCALL;
-            return rc;
-        }
-    }
+    // BIO is disabled for now (see socket.cpp) and the following implementation is
+    // NOT correct since it doesn't handle the EAGAIN event of BIO_flush
+//    BIO* wbio = SSL_get_wbio(ssl);
+//    if (BIO_wpending(wbio) > 0) {
+//        int rc = BIO_flush(wbio);
+//        if (rc <= 0 && BIO_fd_non_fatal_error(errno) == 0) {
+//            // Fatal error during BIO_flush
+//            *ssl_error = SSL_ERROR_SYSCALL;
+//            return rc;
+//        }
+//    }
 #else
     int rc = SSL_flush(ssl);
     if (rc <= 0) {
@@ -2030,7 +1870,7 @@ bool IOBufAsZeroCopyInputStream::Skip(int count) {
     return false;
 }
 
-google::protobuf::int64 IOBufAsZeroCopyInputStream::ByteCount() const {
+int64_t IOBufAsZeroCopyInputStream::ByteCount() const {
     return _byte_count;
 }
 
@@ -2151,7 +1991,7 @@ void IOBufAsZeroCopyOutputStream::BackUp(int count) {
     LOG_IF(FATAL, count != 0) << "BackUp an empty IOBuf";
 }
 
-google::protobuf::int64 IOBufAsZeroCopyOutputStream::ByteCount() const {
+int64_t IOBufAsZeroCopyOutputStream::ByteCount() const {
     return _byte_count;
 }
 
@@ -2262,4 +2102,4 @@ bool IOBufBytesIterator::forward_one_block(const void** data, size_t* size) {
 
 void* fast_memcpy(void *__restrict dest, const void *__restrict src, size_t n) {
     return butil::iobuf::cp(dest, src, n);
-}
+} // namespace butil
\ No newline at end of file
