@@ -20,6 +20,8 @@
 #ifndef BVAR_MULTI_DIMENSION_H
 #define BVAR_MULTI_DIMENSION_H
 
+#include <memory>
+#include <type_traits>
 #include "butil/logging.h"                           // LOG
 #include "butil/macros.h"                            // BAIDU_CASSERT
 #include "butil/scoped_lock.h"                       // BAIDU_SCOPE_LOCK
@@ -35,8 +37,15 @@ namespace bvar {
 // 2. KeyType::value_type must be std::string.
 // 3. KeyType::size() returns the number of labels.
 // 4. KeyType::push_back() adds a label to the end of the container.
-template <typename T, typename KeyType = std::list<std::string>>
+//
+// If `Shared' is false, `get_stats' returns a raw pointer,
+// `delete_stats' and `clear_stats' are not thread safe.
+// If `Shared' is true, `get_stats` returns a shared_ptr,
+// `delete_stats' and `clear_stats' are thread safe.
+// Note: The shared mode may be less performant than the non-shared mode.
+template <typename T, typename KeyType = std::list<std::string>, bool Shared = false>
 class MultiDimension : public MVariable<KeyType> {
+    typedef std::shared_ptr<T> shared_value_type;
 public:
     enum STATS_OP {
         READ_ONLY,
@@ -45,7 +54,7 @@ public:
 
     typedef KeyType key_type;
     typedef T value_type;
-    typedef T* value_ptr_type;
+    typedef typename std::conditional<Shared, shared_value_type, T*>::type value_ptr_type;
     typedef MVariable<key_type> Base;
 
     struct KeyHash {
@@ -102,11 +111,15 @@ public:
     // 2. K::value_type must be able to convert to std::string and butil::StringPiece
     //    through operator std::string() function and operator butil::StringPiece() function.
     // 3. K::value_type must be able to compare with std::string.
+    //
+    // Returns a shared_ptr if `Shared' is true, otherwise returns a raw pointer.
     template <typename K = key_type>
-    T* get_stats(const K& labels_value) {
+    value_ptr_type get_stats(const K& labels_value) {
         return get_stats_impl(labels_value, READ_OR_INSERT);
     }
 
+    // `delete_stats' and `clear_stats' are thread safe
+    // if `Shared' is true, otherwise not.
     // Remove stat so those not count and dump
     template <typename K = key_type>
     void delete_stats(const K& labels_value);
@@ -133,7 +146,7 @@ public:
     // Return real bvar pointer if labels_name exist, NULL otherwise.
     // CAUTION!!! Just For Debug!!!
     template <typename K = key_type>
-    T* get_stats_read_only(const K& labels_value) {
+    value_ptr_type get_stats_read_only(const K& labels_value) {
         return get_stats_impl(labels_value);
     }
 
@@ -141,17 +154,18 @@ public:
     // Return real bvar pointer if labels_name exist, otherwise(not exist) create bvar pointer.
     // CAUTION!!! Just For Debug!!!
     template <typename K = key_type>
-    T* get_stats_read_or_insert(const K& labels_value, bool* do_write = NULL) {
+    value_ptr_type get_stats_read_or_insert(const K& labels_value, bool* do_write = NULL) {
         return get_stats_impl(labels_value, READ_OR_INSERT, do_write);
     }
 #endif
 
 private:
     template <typename K>
-    T* get_stats_impl(const K& labels_value);
+    value_ptr_type get_stats_impl(const K& labels_value);
 
     template <typename K>
-    T* get_stats_impl(const K& labels_value, STATS_OP stats_op, bool* do_write = NULL);
+    value_ptr_type get_stats_impl(
+        const K& labels_value, STATS_OP stats_op, bool* do_write = NULL);
 
     template <typename K>
     static typename std::enable_if<butil::is_same<K, key_type>::value>::type
@@ -162,11 +176,8 @@ private:
     template <typename K>
     static typename std::enable_if<!butil::is_same<K, key_type>::value>::type
     insert_metrics_map(MetricMap& bg, const K& labels_value, op_value_type metric) {
-        key_type labels_value_str;
-        for (auto& label : labels_value) {
-            // key_type::value_type must be able to convert to std::string.
-            labels_value_str.push_back(std::string(label));
-        }
+        // key_type::value_type must be able to convert to std::string.
+        key_type labels_value_str(labels_value.cbegin(), labels_value.cend());
         bg.insert(labels_value_str, metric);
     }
 
@@ -192,7 +203,27 @@ private:
     void delete_stats();
     
     static size_t init_flatmap(MetricMap& bg);
-    
+
+    // If Shared is true, return std::shared_ptr, otherwise return raw pointer.
+    template <bool S = Shared>
+    typename std::enable_if<S, value_ptr_type>::type  new_value() {
+        return std::make_shared<value_type>();
+    }
+    template <bool S = Shared>
+    typename std::enable_if<!S, value_ptr_type>::type  new_value() {
+        return new value_type();
+    }
+
+    // If Shared is true, reset std::shared_ptr, otherwise delete raw pointer.
+    template <bool S = Shared>
+    typename std::enable_if<S>::type delete_value(value_ptr_type& v) {
+        v.reset();
+    }
+    template <bool S = Shared>
+    typename std::enable_if<!S>::type delete_value(value_ptr_type& v) {
+        delete v;
+    }
+
     size_t _max_stats_count;
     MetricMapDBD _metric_map;
 };
